'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    visitor: {
      ImportDeclaration: function ImportDeclaration(path, file) {
        var _ref3;

        var _path$node = path.node;
        var specifiers = _path$node.specifiers;
        var source = _path$node.source;

        if (!t.isStringLiteral(source) || !/^glob:/.test(source.value)) {
          return;
        }

        rejectImportDefaultSpecifier(path, specifiers);

        var fromDir = (0, _path.dirname)(file.file.opts.filename);
        var matches = _glob2.default.sync(getPattern(path, source), {
          // Search relative to the source file, assuming that location is
          // derived correctly.
          cwd: fromDir,
          strict: true
        });

        var prefix = (0, _commonPathPrefix2.default)(matches);
        var suffix = (0, _commonExtname2.default)(matches);

        var lookup = (0, _create2.default)(null);
        var members = [];
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          var _loop = function _loop() {
            var filepath = _step2.value;

            var src = './' + (0, _path.relative)(fromDir, (0, _path.resolve)(fromDir, filepath));
            var subpath = filepath.slice(prefix.length, filepath.length - suffix.length);
            var pieces = subpath.split(_path.sep);
            var member = pieces.map(function (name, index) {
              var id = (0, _identifierfy2.default)(name, {
                prefixReservedWords: pieces.length === 1,
                prefixInvalidIdentifiers: index === 0
              });
              if (id === null) {
                throw path.buildCodeFrameError('Could not generate a valid identifier for \'' + src + '\'. The \'' + name + '\' component could not be converted.');
              }
              return id;
            }).join('$');

            if (lookup[member]) {
              // hyphen conversion means foo-bar and fooBar will collide.
              throw path.buildCodeFrameError('Found colliding members \'' + member + '\'');
            }

            lookup[member] = src;
            members.push(member);
          };

          for (var _iterator2 = (0, _getIterator3.default)(matches), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            _loop();
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        if (specifiers.length === 0) {
          path.replaceWithMultiple(members.map(function (member) {
            return t.importDeclaration([], t.stringLiteral(lookup[member]));
          }));
          return;
        }

        var makeImport = function makeImport(localName, src) {
          return t.importDeclaration([t.importDefaultSpecifier(t.identifier(localName))], t.stringLiteral(src));
        };
        var makeNamespaceObject = function makeNamespaceObject(localName, members) {
          var properties = members.map(function (member) {
            return t.objectProperty(t.identifier(member), t.identifier('_' + localName + '_' + member));
          });
          return t.variableDeclaration('const', [t.variableDeclarator(t.identifier(localName), t.objectExpression(properties))]);
        };
        var freezeNamespaceObject = function freezeNamespaceObject(localName) {
          return t.expressionStatement(t.callExpression(t.memberExpression(t.identifier('Object'), t.identifier('freeze')), [t.identifier(localName)]));
        };

        var replacement = specifiers.map(function (_ref2) {
          var type = _ref2.type;
          var imported = _ref2.imported;
          var localName = _ref2.local.name;

          if (type === 'ImportSpecifier') {
            var importName = imported.name;

            if (!lookup[importName]) {
              throw path.buildCodeFrameError('Could not match import \'' + importName + '\' to a module. Available members are \'' + members.join("', '") + '\'');
            }

            return makeImport(localName, lookup[importName]);
          }

          // Only ImportNamespaceSpecifier can be remaining, since
          // importDefaultSpecifier has previously been rejected.
          return [].concat(members.map(function (member) {
            return makeImport('_' + localName + '_' + member, lookup[member]);
          }), makeNamespaceObject(localName, members), freezeNamespaceObject(localName));
        });
        path.replaceWithMultiple((_ref3 = []).concat.apply(_ref3, (0, _toConsumableArray3.default)(replacement)));
      }
    }
  };
};

var _path = require('path');

var _commonExtname = require('common-extname');

var _commonExtname2 = _interopRequireDefault(_commonExtname);

var _commonPathPrefix = require('common-path-prefix');

var _commonPathPrefix2 = _interopRequireDefault(_commonPathPrefix);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _identifierfy = require('identifierfy');

var _identifierfy2 = _interopRequireDefault(_identifierfy);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getPattern(path, source) {
  var pattern = source.value.replace(/^glob:/, '').trim();
  if (!pattern) {
    throw path.buildCodeFrameError('Missing glob pattern \'' + source.value + '\'');
  }
  if (/^\//.test(pattern)) {
    throw path.buildCodeFrameError('Glob pattern must be relative, was \'' + pattern + '\'');
  }
  return pattern;
}

function rejectImportDefaultSpecifier(path, specifiers) {
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator3.default)(specifiers), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var type = _step.value.type;

      if (type === 'ImportDefaultSpecifier') {
        throw path.buildCodeFrameError('Cannot import the default member');
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}
//# sourceMappingURL=index.js.map